@@ -4,6 +4,7 @@ import inspect
 import re
 import sys
 import textwrap
+import types
 from ast import FunctionDef, Module, stmt
 from typing import Any, AnyStr, Callable, ForwardRef, NewType, TypeVar, get_type_hints
 
@@ -21,8 +22,16 @@ from .version import __version__
 _LOGGER = logging.getLogger(__name__)
 _PYDATA_ANNOTATIONS = {"Any", "AnyStr", "Callable", "ClassVar", "Literal", "NoReturn", "Optional", "Tuple", "Union"}
 
+# types has a bunch of things like ModuleType where ModuleType.__module__ is
+# "builtins" and ModuleType.__name__ is "module", so we have to check for this.
+_TYPES_DICT = {getattr(types, name): name for name in types.__all__}
+# Prefer FunctionType to LambdaType (they are synonymous)
+_TYPES_DICT[types.FunctionType] = "FunctionType"
+
 
 def get_annotation_module(annotation: Any) -> str:
+    if annotation in _TYPES_DICT:
+        return "types"
     if annotation is None:
         return "builtins"
     is_new_type = sys.version_info >= (3, 10) and isinstance(annotation, NewType)
@@ -35,17 +44,22 @@ def get_annotation_module(annotation: Any) -> str:
     raise ValueError(f"Cannot determine the module of {annotation}")
 
 
+def _is_newtype(annotation: Any) -> bool:
+    if sys.version_info < (3, 10):
+        return inspect.isfunction(annotation) and hasattr(annotation, "__supertype__")
+    else:
+        return isinstance(annotation, NewType)
+
+
 def get_annotation_class_name(annotation: Any, module: str) -> str:
     # Special cases
     if annotation is None:
         return "None"
-    elif annotation is Any:
-        return "Any"
-    elif annotation is AnyStr:
+    if annotation is AnyStr:
         return "AnyStr"
-    elif (sys.version_info < (3, 10) and inspect.isfunction(annotation) and hasattr(annotation, "__supertype__")) or (
-        sys.version_info >= (3, 10) and isinstance(annotation, NewType)
-    ):
+    if annotation in _TYPES_DICT:
+        return _TYPES_DICT[annotation]
+    if _is_newtype(annotation):
         return "NewType"
 
     if getattr(annotation, "__qualname__", None):
@@ -139,6 +153,9 @@ def format_annotation(annotation: Any, config: Config) -> str:  # noqa: C901 # t
     if module == "typing_extensions":
         module = "typing"
 
+    if module == "_io":
+        module = "io"
+
     full_name = f"{module}.{class_name}" if module != "builtins" else class_name
     fully_qualified: bool = getattr(config, "typehints_fully_qualified", False)
     prefix = "" if fully_qualified or full_name == class_name else "~"
@@ -168,18 +185,20 @@ def format_annotation(annotation: Any, config: Config) -> str:  # noqa: C901 # t
     elif full_name in ("typing.Union", "types.UnionType") and type(None) in args:
         if len(args) == 2:
             full_name = "typing.Optional"
+            role = "data"
             args = tuple(x for x in args if x is not type(None))  # noqa: E721
         else:
             simplify_optional_unions: bool = getattr(config, "simplify_optional_unions", True)
             if not simplify_optional_unions:
                 full_name = "typing.Optional"
+                role = "data"
                 args_format = f"\\[:py:data:`{prefix}typing.Union`\\[{{}}]]"
                 args = tuple(x for x in args if x is not type(None))  # noqa: E721
-    elif full_name == "typing.Callable" and args and args[0] is not ...:
+    elif full_name in ("typing.Callable", "collections.abc.Callable") and args and args[0] is not ...:
         fmt = [format_annotation(arg, config) for arg in args]
         formatted_args = f"\\[\\[{', '.join(fmt[:-1])}], {fmt[-1]}]"
     elif full_name == "typing.Literal":
-        formatted_args = f"\\[{', '.join(repr(arg) for arg in args)}]"
+        formatted_args = "\\[{}]".format(", ".join(f"``{arg!r}``" for arg in args))
     elif full_name == "types.UnionType":
         return " | ".join([format_annotation(arg, config) for arg in args])
 
@@ -609,7 +628,7 @@ def _inject_types_to_docstring(
 
                 lines.insert(insert_index, type_annotation)
 
-    if "return" in type_hints and not inspect.isclass(original_obj):
+    if "return" in type_hints and not inspect.isclass(original_obj) and not inspect.isdatadescriptor(original_obj):
         if what == "method" and name.endswith(".__init__"):  # avoid adding a return type for data class __init__
             return
         formatted_annotation = format_annotation(type_hints["return"], app.config)
@@ -623,6 +642,10 @@ def _inject_types_to_docstring(
                     insert_index = None
                     break
                 insert_index = at
+            elif line.startswith(".."):
+                # Make sure that rtype comes before any usage or examples section
+                insert_index = at
+                break
 
         if insert_index is not None and app.config.typehints_document_rtype:
             if insert_index == len(lines):  # ensure that :rtype: doesn't get joined with a paragraph of text
