@@ -4,6 +4,7 @@ import collections.abc
 import pathlib
 import re
 import sys
+import types
 import typing
 from functools import cmp_to_key
 from io import StringIO
@@ -111,12 +112,33 @@ class HintedMethods:
 
 PY310_PLUS = sys.version_info >= (3, 10)
 
+if sys.version_info >= (3, 9):
+    AbcCallable = collections.abc.Callable  # type: ignore[type-arg]
+else:
+    # Hacks to make it work the same in old versions.
+    # We could also set AbcCallable = typing.Callable and x fail the tests that
+    # use AbcCallable when in versions less than 3.9.
+    class MyGenericAlias(typing._VariadicGenericAlias, _root=True):  # noqa: SC200
+        def __getitem__(self, params):
+            result = super().__getitem__(params)
+            # Make a copy so we don't change the name of a cached annotation
+            result = result.copy_with(result.__args__)
+            result.__module__ = "collections.abc"
+            return result
+
+    AbcCallable = MyGenericAlias(collections.abc.Callable, (), special=True)
+    AbcCallable.__module__ = "collections.abc"
+
 
 @pytest.mark.parametrize(
     ("annotation", "module", "class_name", "args"),
     [
         pytest.param(str, "builtins", "str", (), id="str"),
         pytest.param(None, "builtins", "None", (), id="None"),
+        pytest.param(ModuleType, "types", "ModuleType", (), id="ModuleType"),
+        pytest.param(FunctionType, "types", "FunctionType", (), id="FunctionType"),
+        pytest.param(types.CodeType, "types", "CodeType", (), id="CodeType"),
+        pytest.param(types.CoroutineType, "types", "CoroutineType", (), id="CoroutineType"),
         pytest.param(Any, "typing", "Any", (), id="Any"),
         pytest.param(AnyStr, "typing", "AnyStr", (), id="AnyStr"),
         pytest.param(Dict, "typing", "Dict", (), id="Dict"),
@@ -128,6 +150,13 @@ PY310_PLUS = sys.version_info >= (3, 10)
         pytest.param(Callable, "typing", "Callable", (), id="Callable"),
         pytest.param(Callable[..., str], "typing", "Callable", (..., str), id="Callable_returntype"),
         pytest.param(Callable[[int, str], str], "typing", "Callable", (int, str, str), id="Callable_all_types"),
+        pytest.param(
+            AbcCallable[[int, str], str],  # type: ignore[misc]
+            "collections.abc",
+            "Callable",
+            (int, str, str),
+            id="collections.abc.Callable_all_types",
+        ),
         pytest.param(Pattern, "typing", "Pattern", (), id="Pattern"),
         pytest.param(Pattern[str], "typing", "Pattern", (str,), id="Pattern_parametrized"),
         pytest.param(Match, "typing", "Match", (), id="Match"),
@@ -146,9 +175,10 @@ PY310_PLUS = sys.version_info >= (3, 10)
     ],
 )
 def test_parse_annotation(annotation: Any, module: str, class_name: str, args: tuple[Any, ...]) -> None:
-    assert get_annotation_module(annotation) == module
-    assert get_annotation_class_name(annotation, module) == class_name
-    assert get_annotation_args(annotation, module, class_name) == args
+    got_mod = get_annotation_module(annotation)
+    got_cls = get_annotation_class_name(annotation, module)
+    got_args = get_annotation_args(annotation, module, class_name)
+    assert (got_mod, got_cls, got_args) == (module, class_name, args)
 
 
 @pytest.mark.parametrize(
@@ -156,6 +186,9 @@ def test_parse_annotation(annotation: Any, module: str, class_name: str, args: t
     [
         (str, ":py:class:`str`"),
         (int, ":py:class:`int`"),
+        (StringIO, ":py:class:`~io.StringIO`"),
+        (FunctionType, ":py:class:`~types.FunctionType`"),
+        (ModuleType, ":py:class:`~types.ModuleType`"),
         (type(None), ":py:obj:`None`"),
         (type, ":py:class:`type`"),
         (collections.abc.Callable, ":py:class:`~collections.abc.Callable`"),
@@ -203,6 +236,7 @@ def test_parse_annotation(annotation: Any, module: str, class_name: str, args: t
             ),
         ),
         (Optional[str], ":py:data:`~typing.Optional`\\[:py:class:`str`]"),
+        (Union[str, None], ":py:data:`~typing.Optional`\\[:py:class:`str`]"),
         (
             Optional[Union[str, bool]],
             ":py:data:`~typing.Union`\\[:py:class:`str`, " ":py:class:`bool`, :py:obj:`None`]",
@@ -223,6 +257,10 @@ def test_parse_annotation(annotation: Any, module: str, class_name: str, args: t
             ":py:data:`~typing.Callable`\\[\\[:py:class:`~typing.TypeVar`\\(``T``)],"
             " :py:class:`~typing.TypeVar`\\(``T``)]",
         ),
+        (
+            AbcCallable[[int, str], bool],  # type: ignore[misc]
+            ":py:class:`~collections.abc.Callable`\\[\\[:py:class:`int`, " ":py:class:`str`], :py:class:`bool`]",
+        ),
         (Pattern, ":py:class:`~typing.Pattern`"),
         (Pattern[str], ":py:class:`~typing.Pattern`\\[:py:class:`str`]"),
         (IO, ":py:class:`~typing.IO`"),
@@ -359,7 +397,7 @@ def test_format_annotation(inv: Inventory, annotation: Any, expected_result: str
     [
         ("ClassVar", int, ":py:data:`~typing.ClassVar`\\[:py:class:`int`]"),
         ("NoReturn", None, ":py:data:`~typing.NoReturn`"),
-        ("Literal", ("a", 1), ":py:data:`~typing.Literal`\\['a', 1]"),
+        ("Literal", ("a", 1), ":py:data:`~typing.Literal`\\[``'a'``, ``1``]"),
         ("Type", None, ":py:class:`~typing.Type`"),
         ("Type", (A,), f":py:class:`~typing.Type`\\[:py:class:`~{__name__}.A`]"),
     ],
@@ -545,9 +583,6 @@ def test_sphinx_output(
 
               Property docstring
 
-              Return type:
-                 "str"
-
            static a_staticmethod(x, y, z=None)
 
               Staticmethod docstring.
@@ -722,6 +757,17 @@ def test_sphinx_output(
 
            Parameters:
               **x** ("Mailbox") -- function
+
+        dummy_module.func_with_examples()
+
+           A docstring.
+
+           Return type:
+              "int"
+
+           -[ Examples ]-
+
+           Here are a couple of examples of how to use this function.
         """
         expected_contents = dedent(expected_contents).format(**format_args).replace("–", "--")
         assert text_contents == maybe_fix_py310(expected_contents)
